//THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
//ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
//THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
//
// Copyright  1994-1995  Microsoft Corporation.  All Rights Reserved.
//
//	PROGRAM:	Generic.c
//
//	PURPOSE:	Illustrates the 'minimum' functionality of a well-behaved Win32 application..
//
//	PLATFORMS:	Windows 95, Windows NT, Win32s
//
//	FUNCTIONS:	
//		WinMain() - calls initialization function, processes message loop
//		InitApplication() - Initializes window data nd registers window
//		InitInstance() -saves instance handle and creates main window
//		WindProc() Processes messages
//		About() - Process menssages for "About" dialog box
//		MyRegisterClass() - Registers the application's window class
//		CenterWindow() -  Centers one window over another
//
//	SPECIAL INSTRUCTIONS: N/A
//
#define APPNAME "Generic"

// Windows Header Files:
#include <windows.h>

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <stdio.h>

// Local Header Files
#include "include\generic.h"
#include "include\interface.h"
#include "include\resource.h"
//#include "include\mazeprivate.h"
#include "include\interfaceprivate.h"

// Makes it easier to determine appropriate code paths:
#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif
#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95 (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

// Global Variables:

HINSTANCE hInst; // current instance
HWND hWndGlobal, hConsoleWin; // main window handle
char szAppName[] = APPNAME; // The name of this application
char szTitle[]   = APPNAME": Sample Application"; // The title bar text
int localSonarHidden = 0;
CRITICAL_SECTION SerialLock, SonarLock, Input1Lock, Input2Lock, CommandLock,
	PrintLock;
int UpArrow, DownArrow, LeftArrow, RightArrow, SpaceBar, EnterKey, ZeroNow,
	SonarState, FooState, BarState;
int LineAvailable = 0, LineWait = 0, InputBarrier = 0;
char LineBuf[1024];
HPEN hMazeErasePen, hMazeDrawPen, hErasePen, hDrawPen, hKeyErasePen, 
	hKeyDrawPen;
HBRUSH hMazeEraseBrush;
HWND hMazeWnd;
HDC hMazeDC;

extern HWND hKeybdWnd, hZeroButton, hSonarButton, hSonarWnd, hKeybdWnd;
extern HDC hSonarDC, hKeybdDC;
extern int SonarsActive, SonarScaleFactor, SonarWinShown;

// Foward declarations of functions included in this code module:

ATOM MyRegisterClass(CONST WNDCLASS*);
BOOL InitApplication(HINSTANCE);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
BOOL CenterWindow (HWND, HWND);
DWORD RobotThread(LPDWORD foo);
LRESULT CALLBACK SerialErrFn(HWND hDlg, UINT message, WPARAM wParam,
							 LPARAM lParam);
LRESULT CALLBACK ConsoleCallBk(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void ExitFunc();
HWND CreateMazeWindow();
HWND CreateSonarWindow();
HWND CreateKeybdWindow();
int RobotMain();

//
//  FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
//
//  PURPOSE: Entry point for the application.
//
//  COMMENTS:
//
//	This function initializes the application and processes the
//	message loop.
//
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{

	MSG msg;
	HANDLE hAccelTable, hRobotThread;
	DWORD RobotThreadID;
	BOOL res;

	// register handler to turn off sonars when the program exits.
	atexit(( void ( __cdecl *)( void )) ExitFunc);

	InitializeCriticalSection(&SerialLock);
	InitializeCriticalSection(&SonarLock);
	InitializeCriticalSection(&Input1Lock);
	InitializeCriticalSection(&Input2Lock);
	InitializeCriticalSection(&CommandLock);
	InitializeCriticalSection(&PrintLock);

	// need this to use trackbar.
	InitCommonControls();  
	
	if (!hPrevInstance) {
		// Perform instance initialization:
		if (!InitApplication(hInstance)) {
			return (FALSE);
		}
	}

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) {
		return (FALSE);
	}

	hAccelTable = LoadAccelerators (hInstance, szAppName);

	if (OpenSerial()) {
		// some other program owns the serial port.
		DialogBox(hInst, "SERIALERR", hWndGlobal, SerialErrFn);
		exit(-2);
	}
	
	
	CreateKeybdWindow();
	CreateSonarWindow();
	CreateMazeWindow();

	UnhideSonars();
	
	hRobotThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) RobotThread,
							   NULL, 0, &RobotThreadID);
	
	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	EnterCriticalSection(&CommandLock);
	EnterCriticalSection(&SerialLock);
	res = TerminateThread(hRobotThread, 0);
	LeaveCriticalSection(&SerialLock);
	LeaveCriticalSection(&CommandLock);

	return (msg.wParam);

	lpCmdLine; // This will prevent 'unused formal parameter' warnings
}

void ExitFunc()
{
	if (SonarsActive) {
		SonarsOff();
	}
}

DWORD RobotThread(LPDWORD foo)
{
	exit(RobotMain());
	return 1;
}

// callback for serial port error dialog.  (when serial port can't be opened.)
LRESULT CALLBACK SerialErrFn(HWND hDlg, UINT message, WPARAM wParam,
							 LPARAM lParam)
{

	switch (message) {
        case WM_INITDIALOG:
			ShowWindow (hDlg, SW_SHOW);
			return (TRUE);

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK) {
				EndDialog(hDlg, TRUE);
				return (TRUE);
			}
			break;
	}

    return FALSE;
}

// returns the last character of the string
char LastChar(char *str)
{
	return(str[strlen(str)-1]);
}

// callback function for the Robot Console
LRESULT CALLBACK ConsoleCallBk(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent, done;
	
	switch (message) {
	    case WM_CLOSE:
			PostQuitMessage(0);
			break;

	    case WM_HSCROLL:
			// user moved the sonar mag slider
			SonarScaleFactor = SendDlgItemMessage(hDlg, IDC_SLIDER1,
												  TBM_GETPOS, 0, 0);
			break;
			
        case WM_INITDIALOG:
			SetWindowPos (hDlg, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
			SendDlgItemMessage(hDlg, IDC_SLIDER1, TBM_SETRANGE, TRUE,
							   MAKELONG(1,8));
			SendDlgItemMessage(hDlg, IDC_SLIDER1, TBM_SETPAGESIZE, 0, 1);
			SendDlgItemMessage(hDlg, IDC_SLIDER1, TBM_SETSEL, FALSE,
							   MAKELONG(1,8));
			SendDlgItemMessage(hDlg, IDC_SLIDER1, TBM_SETPOS, TRUE, 1);
			
			ShowWindow (hDlg, SW_SHOW);
			return (TRUE);

		case WM_COMMAND:
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);
			switch (wmId) {
			    case IDC_SONARCHECK:
					SonarHideToggle();
					break;

			    case IDEXIT:
				    PostQuitMessage(0);
					break;

			    case IDC_FOOBUTTON:
					// someone clicked the Foo button.
					// FooState is zeroed in the user's thread by
					// GetFooClick()
					FooState = 1;
					break;
					
			    case IDC_BARBUTTON:
					// someone clicked the Bar button.
					// FooState is zeroed in the user's thread by
					// GetBarClick()
					BarState = 1;
					break;
					
			    case IDC_EDIT1:
					break;
					
			    case IDC_EDIT2:
					switch (wmEvent) {
					    case EN_CHANGE:
							GetDlgItemText(hDlg, IDC_EDIT2, LineBuf, 1023);
							if (LastChar(LineBuf) == '\n') {
								// if user hit <Enter>, allow
								// GetInputBoxText to copy buffer.

								EnterCriticalSection(&Input1Lock);
								if (LineWait) {
									LineWait = 0;
									LineAvailable = 1;
									LeaveCriticalSection(&Input1Lock);

									done = 0;
									while (!done) {
										EnterCriticalSection(&Input2Lock);
										done = (InputBarrier == 1);
										LeaveCriticalSection(&Input2Lock);
									}
									EnterCriticalSection(&Input2Lock);
									InputBarrier = 0;
									LeaveCriticalSection(&Input2Lock);
								} else {
									LeaveCriticalSection(&Input1Lock);
								}

							}
							break;
							
					}
					break;
			}
			break;
	}

    return FALSE;
}

//
//  FUNCTION: MyRegisterClass(CONST WNDCLASS*)
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
// 	This function and its usage is only necessary if you want this code
// 	to be compatible with Win32 systems prior to the 'RegisterClassEx'
//	function that was added to Windows 95. It is important to call this function
// 	so that the application will get 'well formed' small icons associated
// 	with it.
//
ATOM MyRegisterClass(CONST WNDCLASS *lpwc)
{
	HANDLE  hMod;
	FARPROC proc;
	WNDCLASSEX wcex;

	hMod = GetModuleHandle ("USER32");
	if (hMod != NULL) {

#if defined (UNICODE)
		proc = GetProcAddress (hMod, "RegisterClassExW");
#else
		proc = GetProcAddress (hMod, "RegisterClassExA");
#endif

		if (proc != NULL) {

			wcex.style         = lpwc->style;
			wcex.lpfnWndProc   = lpwc->lpfnWndProc;
			wcex.cbClsExtra    = lpwc->cbClsExtra;
			wcex.cbWndExtra    = lpwc->cbWndExtra;
			wcex.hInstance     = lpwc->hInstance;
			wcex.hIcon         = lpwc->hIcon;
			wcex.hCursor       = lpwc->hCursor;
			wcex.hbrBackground = lpwc->hbrBackground;
            			wcex.lpszMenuName  = lpwc->lpszMenuName;
			wcex.lpszClassName = lpwc->lpszClassName;

			// Added elements for Windows 95:
			wcex.cbSize = sizeof(WNDCLASSEX);
			wcex.hIconSm = LoadIcon(wcex.hInstance, "SMALL");
			
			return (*proc)(&wcex);//return RegisterClassEx(&wcex);
		}
	}
	return (RegisterClass(lpwc));
}


//
//  FUNCTION: InitApplication(HANDLE)
//
//  PURPOSE: Initializes window data and registers window class 
//
//  COMMENTS:
//
//       In this function, we initialize a window class by filling out a data
//       structure of type WNDCLASS and calling either RegisterClass or 
//       the internal MyRegisterClass.
//
BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;
    HWND      hwnd;
	ATOM atm;

    // Win32 will always set hPrevInstance to NULL, so lets check
    // things a little closer. This is because we only want a single
    // version of this app to run at a time
    hwnd = FindWindow (szAppName, NULL);
    if (hwnd) {
        // We found another version of ourself. Lets defer to it:
        if (IsIconic(hwnd)) {
            ShowWindow(hwnd, SW_RESTORE);
        }
        SetForegroundWindow (hwnd);

        // If this app actually had any functionality, we would
        // also want to communicate any action that our 'twin'
        // should now perform based on how the user tried to
        // execute us.
        return FALSE;
        }

        // Fill in window class structure with parameters that describe
        // the main window.
        wc.style         = CS_HREDRAW | CS_VREDRAW;
        wc.lpfnWndProc   = (WNDPROC)WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = hInstance;
        wc.hIcon         = LoadIcon (hInstance, szAppName);
        wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);

        // Since Windows95 has a slightly different recommended
        // format for the 'Help' menu, lets put this in the alternate menu like this:

/*        if (IS_WIN95) {
	wc.lpszMenuName  = "WIN95";
        } else {
	wc.lpszMenuName  = szAppName;
        }
		*/

		wc.lpszMenuName = NULL;
		
		wc.lpszClassName = szAppName;

        // Register the window class and return success/failure code.
        if (IS_WIN95) {
	atm = MyRegisterClass(&wc);
        } else {
	atm = RegisterClass(&wc);
        }

		wc.lpszMenuName = NULL;
		wc.lpszClassName = "Sonar Display";

		RegisterClass(&wc);

		return atm;
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window 
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;
	
	hInst = hInstance; // Store instance handle in our global variable

	hWndGlobal = NULL;
	hWnd = hConsoleWin = CreateDialog(hInstance, "RCONSOLE",
												   NULL, ConsoleCallBk);

	if (!hWnd) {
		return (FALSE);
	}

	return (TRUE);
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  MESSAGES:
//
//	WM_COMMAND - process the application menu
//	WM_PAINT - Paint the main window
//	WM_DESTROY - post a quit message and return
//    WM_DISPLAYCHANGE - message sent to Plug & Play systems when the display changes
//    WM_RBUTTONDOWN - Right mouse click -- put up context menu here if appropriate
//    WM_NCRBUTTONUP - User has clicked the right button on the application's system menu
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static int i=0;
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	BOOL bGotHelp;

	switch (message) { 

	    case WM_KEYDOWN:
			if (hWnd == hKeybdWnd) {
				switch((int) wParam) {
			        case VK_UP: UpArrow = 3; break;
			        case VK_DOWN: DownArrow = 3; break;
			        case VK_LEFT: LeftArrow = 3; break;
			        case VK_RIGHT: RightArrow = 3; break;
			        case VK_SPACE: SpaceBar = 3; break;
			        case VK_RETURN: EnterKey = 3; break;
						// set bits 0 and 1 high.
				}					
			}
			break;
			
	    case WM_KEYUP:
			if (hWnd == hKeybdWnd) {
				switch((int) wParam) {
			        case VK_UP: UpArrow = UpArrow & 2; break;
			        case VK_DOWN: DownArrow = DownArrow & 2; break;
			        case VK_LEFT: LeftArrow = LeftArrow & 2; break;
			        case VK_RIGHT: RightArrow = RightArrow & 2; break;
			        case VK_SPACE: SpaceBar = SpaceBar & 2; break;
			        case VK_RETURN: EnterKey = EnterKey & 2; break;
						// set bit 1 low.  bit zero is set low in KeybdControl.
				}					
			}
			break;

	    case WM_CLOSE:
			if (hWnd == hKeybdWnd) {
				HideKeybd();
			}
			break;
				
		case WM_COMMAND:
			wmId    = LOWORD(wParam); // Remember, these are...
			wmEvent = HIWORD(wParam); // ...different for Win32!

			if (wmEvent == BN_CLICKED) {
				if ((HWND) lParam == hZeroButton) {
					ZeroNow = 1;
					SetFocus(hKeybdWnd);  // this is necessary to remove focus
					                      // from the Zero button.
					                      // hKeybdWnd doesn't receive keyboard
					                      // events without focus.
				} else if ((HWND) lParam == hSonarButton) {
					SonarState = 1 - SonarState;
					SetFocus(hKeybdWnd);  // same as above.
				}
			}
			
			//Parse the menu selections:
			switch (wmId) {

				case IDM_ABOUT:
					DialogBox(hInst, "AboutBox", hWnd, (DLGPROC)About);
					break;

				case IDM_EXIT:
					DestroyWindow (hWnd);
					break;

                		case IDM_HELPTOPICS: // Only called in Windows 95
                    		bGotHelp = WinHelp (hWnd, APPNAME".HLP", HELP_FINDER,(DWORD)0);
                    		if (!bGotHelp) {
                        		MessageBox (GetFocus(),"Unable to activate help",
                            	szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPCONTENTS: // Not called in Windows 95
                    		bGotHelp = WinHelp (hWnd, APPNAME".HLP", HELP_CONTENTS,(DWORD)0);
		                 if (!bGotHelp) {
           		            MessageBox (GetFocus(),"Unable to activate help",
                      	      szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPSEARCH: // Not called in Windows 95
 		                 if (!WinHelp(hWnd, APPNAME".HLP", HELP_PARTIALKEY,
				 		(DWORD)(LPSTR)"")) {
						MessageBox (GetFocus(), "Unable to activate help",
							szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				case IDM_HELPHELP: // Not called in Windows 95
					if(!WinHelp(hWnd, (LPSTR)NULL, HELP_HELPONHELP, 0)) {
						MessageBox (GetFocus(), "Unable to activate help",
							szAppName, MB_OK|MB_ICONHAND);
					}
					break;

				// Here are all the other possible menu options,
				// all of these are currently disabled:
				case IDM_NEW:
				case IDM_OPEN:
				case IDM_SAVE:
				case IDM_SAVEAS:
				case IDM_UNDO:
				case IDM_CUT:
				case IDM_COPY:
				case IDM_PASTE:
				case IDM_LINK:
				case IDM_LINKS:

				default:
					return (DefWindowProc(hWnd, message, wParam, lParam));
			}
			break;

		case WM_DISPLAYCHANGE: // Only comes through on plug'n'play systems
		{
			SIZE szScreen;
			BOOL fChanged = (BOOL)wParam;

			szScreen.cx = LOWORD(lParam);
			szScreen.cy = HIWORD(lParam);
			
			if (fChanged) {
				// The display 'has' changed. szScreen reflects the
				// new size.
				MessageBox (GetFocus(), "Display Changed", szAppName, 0);
			} else {
				// The display 'is' changing. szScreen reflects the
				// original size.
				MessageBeep(0);
			}
		}
		break;

		case WM_PAINT:
			hdc = BeginPaint (hWnd, &ps);
			if (hWnd == hMazeWnd) {
				//RedrawMazeWindow();
				;
			}
			EndPaint (hWnd, &ps);
			break;        

		case WM_DESTROY:
			// Tell WinHelp we don't need it any more...
	            WinHelp (hWnd, APPNAME".HLP", HELP_QUIT,(DWORD)0);
			PostQuitMessage(0);
			break;

		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}

//
//  FUNCTION: About(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for "About" dialog box
// 		This version allows greater flexibility over the contents of the 'About' box,
// 		by pulling out values from the 'Version' resource.
//
//  MESSAGES:
//
//	WM_INITDIALOG - initialize dialog box
//	WM_COMMAND    - Input received
//
//
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static  HFONT hfontDlg;		// Font for dialog text
	static	HFONT hFinePrint;	// Font for 'fine print' in dialog
	DWORD   dwVerInfoSize;		// Size of version information block
	LPSTR   lpVersion;			// String pointer to 'version' text
	DWORD   dwVerHnd=0;			// An 'ignored' parameter, always '0'
	UINT    uVersionLen;
	WORD    wRootLen;
	BOOL    bRetCode;
	int     i;
	char    szFullPath[256];
	char    szResult[256];
	char    szGetName[256];
	DWORD	dwVersion;
	char	szVersion[40];
	DWORD	dwResult;

	switch (message) {
        case WM_INITDIALOG:
			ShowWindow (hDlg, SW_HIDE);
			hfontDlg = CreateFont(14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
				VARIABLE_PITCH | FF_SWISS, "");
			hFinePrint = CreateFont(11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
				VARIABLE_PITCH | FF_SWISS, "");
			CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));
			GetModuleFileName (hInst, szFullPath, sizeof(szFullPath));

			// Now lets dive in and pull out the version information:
			dwVerInfoSize = GetFileVersionInfoSize(szFullPath, &dwVerHnd);
			if (dwVerInfoSize) {
				LPSTR   lpstrVffInfo;
				HANDLE  hMem;
				hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
				lpstrVffInfo  = GlobalLock(hMem);
				GetFileVersionInfo(szFullPath, dwVerHnd, dwVerInfoSize, lpstrVffInfo);
				// The below 'hex' value looks a little confusing, but
				// essentially what it is, is the hexidecimal representation
				// of a couple different values that represent the language
				// and character set that we are wanting string values for.
				// 040904E4 is a very common one, because it means:
				//   US English, Windows MultiLingual characterset
				// Or to pull it all apart:
				// 04------        = SUBLANG_ENGLISH_USA
				// --09----        = LANG_ENGLISH
				// ----04E4 = 1252 = Codepage for Windows:Multilingual
				lstrcpy(szGetName, "\\StringFileInfo\\040904E4\\");	 
				wRootLen = lstrlen(szGetName); // Save this position
			
				// Set the title of the dialog:
				lstrcat (szGetName, "ProductName");
				bRetCode = VerQueryValue((LPVOID)lpstrVffInfo,
					(LPSTR)szGetName,
					(LPVOID)&lpVersion,
					(UINT *)&uVersionLen);
				lstrcpy(szResult, "About ");
				lstrcat(szResult, lpVersion);
				SetWindowText (hDlg, szResult);

				// Walk through the dialog items that we want to replace:
				for (i = DLG_VERFIRST; i <= DLG_VERLAST; i++) {
					GetDlgItemText(hDlg, i, szResult, sizeof(szResult));
					szGetName[wRootLen] = (char)0;
					lstrcat (szGetName, szResult);
					uVersionLen   = 0;
					lpVersion     = NULL;
					bRetCode      =  VerQueryValue((LPVOID)lpstrVffInfo,
						(LPSTR)szGetName,
						(LPVOID)&lpVersion,
						(UINT *)&uVersionLen);

					if ( bRetCode && uVersionLen && lpVersion) {
					// Replace dialog item text with version info
						lstrcpy(szResult, lpVersion);
						SetDlgItemText(hDlg, i, szResult);
					} else {
						dwResult = GetLastError();
						wsprintf (szResult, "Error %lu", dwResult);
						SetDlgItemText (hDlg, i, szResult);
					}
					SendMessage (GetDlgItem (hDlg, i), WM_SETFONT, 
						(UINT)((i==DLG_VERLAST)?hFinePrint:hfontDlg),
						TRUE);
				} // for (i = DLG_VERFIRST; i <= DLG_VERLAST; i++)


				GlobalUnlock(hMem);
				GlobalFree(hMem);

			} else {
				// No version information available.
			} // if (dwVerInfoSize)

            SendMessage (GetDlgItem (hDlg, IDC_LABEL), WM_SETFONT,
				(WPARAM)hfontDlg,(LPARAM)TRUE);

			// We are  using GetVersion rather then GetVersionEx
			// because earlier versions of Windows NT and Win32s
			// didn't include GetVersionEx:
			dwVersion = GetVersion();

			if (dwVersion < 0x80000000) {
				// Windows NT
				wsprintf (szVersion, "Microsoft Windows NT %u.%u (Build: %u)",
					(DWORD)(LOBYTE(LOWORD(dwVersion))),
					(DWORD)(HIBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIWORD(dwVersion)) );
			} else if (LOBYTE(LOWORD(dwVersion))<4) {
				// Win32s
                wsprintf (szVersion, "Microsoft Win32s %u.%u (Build: %u)",
					(DWORD)(LOBYTE(LOWORD(dwVersion))),
					(DWORD)(HIBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIWORD(dwVersion) & ~0x8000) );
			} else {
				// Windows 95
                wsprintf (szVersion, "Microsoft Windows 95 %u.%u",
                    (DWORD)(LOBYTE(LOWORD(dwVersion))),
                    (DWORD)(HIBYTE(LOWORD(dwVersion))) );
			}

		    SetWindowText (GetDlgItem(hDlg, IDC_OSVERSION), szVersion);
			ShowWindow (hDlg, SW_SHOW);
			return (TRUE);

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, TRUE);
				DeleteObject (hfontDlg);
				DeleteObject (hFinePrint);
				return (TRUE);
			}
			break;
	}

    return FALSE;
}

//
//   FUNCTION: CenterWindow(HWND, HWND)
//
//   PURPOSE: Centers one window over another. 
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
// 		This functionwill center one window over another ensuring that 
//		the placement of the window is within the 'working area', meaning 
//		that it is both within the display limits of the screen, and not 
//		obscured by the tray or other framing elements of the desktop.
BOOL CenterWindow (HWND hwndChild, HWND hwndParent)
{
	RECT    rChild, rParent, rWorkArea;
	int     wChild, hChild, wParent, hParent;
	int     xNew, yNew;
	BOOL 	bResult;

	// Get the Height and Width of the child window
	GetWindowRect (hwndChild, &rChild);
	wChild = rChild.right - rChild.left;
	hChild = rChild.bottom - rChild.top;

	// Get the Height and Width of the parent window
	GetWindowRect (hwndParent, &rParent);
	wParent = rParent.right - rParent.left;
	hParent = rParent.bottom - rParent.top;

	// Get the limits of the 'workarea'
	bResult = SystemParametersInfo(
		SPI_GETWORKAREA,	// system parameter to query or set
		sizeof(RECT),
		&rWorkArea,
		0);
	if (!bResult) {
		rWorkArea.left = rWorkArea.top = 0;
		rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
		rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
	}

	// Calculate new X position, then adjust for workarea
	xNew = rParent.left + ((wParent - wChild) /2);
	if (xNew < rWorkArea.left) {
		xNew = rWorkArea.left;
	} else if ((xNew+wChild) > rWorkArea.right) {
		xNew = rWorkArea.right - wChild;
	}

	// Calculate new Y position, then adjust for workarea
	yNew = rParent.top  + ((hParent - hChild) /2);
	if (yNew < rWorkArea.top) {
		yNew = rWorkArea.top;
	} else if ((yNew+hChild) > rWorkArea.bottom) {
		yNew = rWorkArea.bottom - hChild;
	}

	// Set it, and return
	return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

// create the window for maze graphics.
HWND CreateMazeWindow()
{
	hMazeWnd = CreateWindow(
    "Sonar Display",	// pointer to registered class name
    "Maze",	// pointer to window name
//    WS_POPUP | WS_BORDER,	// window style
    WS_OVERLAPPED | WS_BORDER | WS_MINIMIZEBOX | WS_SYSMENU,	// window style
    0,	// horizontal position of window
    300,	// vertical position of window
    50,	// window width
    50,	// window height
    hWndGlobal,	// handle to parent or owner window
    NULL,	// handle to menu or child-window identifier
    hInst,	// handle to application instance
    NULL 	// pointer to window-creation data
   );

	hMazeDC = GetDC(hMazeWnd);

	hMazeErasePen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
	hMazeDrawPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));

	hMazeEraseBrush = CreateSolidBrush(RGB(255, 255, 255));
	
	return hMazeWnd;
}

// create the window for the sonar display.
HWND CreateSonarWindow()
{
	RECT winsize;

	winsize.left = 50;
	winsize.right = SONARWINSIZE + 50;
	winsize.top = 50;
	winsize.bottom = SONARWINSIZE + 50;

	if (!AdjustWindowRect(&winsize, WS_OVERLAPPEDWINDOW, FALSE))
		return NULL;

	hSonarWnd = CreateWindow(
    "Sonar Display",	// pointer to registered class name
    "Sonars",	// pointer to window name
//    WS_POPUP | WS_BORDER,	// window style
    WS_OVERLAPPED | WS_BORDER | WS_MINIMIZEBOX | WS_SYSMENU,	// window style
    0,	// horizontal position of window
    300,	// vertical position of window
    winsize.right - winsize.left,	// window width
    winsize.bottom - winsize.top,	// window height
    hWndGlobal,	// handle to parent or owner window
    NULL,	// handle to menu or child-window identifier
    hInst,	// handle to application instance
    NULL 	// pointer to window-creation data
   );

	hSonarDC = GetDC(hSonarWnd);
	hErasePen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
	hDrawPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));
	
	SonarWinShown = 0;
	return hSonarWnd;
}

// create window for keyboard control
HWND CreateKeybdWindow()
{
	RECT winsize;

	winsize.left = 50;
	winsize.right = SONARWINSIZE + 50;
	winsize.top = 50;
	winsize.bottom = SONARWINSIZE + 50 + 24 + 2;

	if (!AdjustWindowRect(&winsize, WS_OVERLAPPEDWINDOW, FALSE))
		return NULL;

	hKeybdWnd = CreateWindow(
    "Sonar Display",	// pointer to registered class name
    "Keyboard Control",	// pointer to window name
//    WS_POPUP | WS_BORDER,	// window style
    WS_OVERLAPPED | WS_BORDER | WS_SYSMENU | WS_MINIMIZEBOX,	// window style
    300,	// horizontal position of window
    300,	// vertical position of window
    winsize.right - winsize.left,	// window width
    winsize.bottom - winsize.top,	// window height
    hWndGlobal,	// handle to parent or owner window
    NULL,	// handle to menu or child-window identifier
    hInst,	// handle to application instance
    NULL 	// pointer to window-creation data
   );

	hZeroButton = CreateWindow(
    "BUTTON",	// pointer to registered class name
    "Zero",	// pointer to window name
//    WS_VISIBLE | BS_PUSHBUTTON,	// window style
    WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON,	// window style
    2,	// horizontal position of window
    2,	// vertical position of window
    52,	// window width
    24,	// window height
    hKeybdWnd,	// handle to parent or owner window
    NULL,	// handle to menu or child-window identifier
    hInst,	// handle to application instance
    NULL 	// pointer to window-creation data
   );
   
	hSonarButton = CreateWindow(
    "BUTTON",	// pointer to registered class name
    "Sonars",	// pointer to window name
//    WS_VISIBLE | BS_PUSHBUTTON,	// window style
    WS_VISIBLE | WS_CHILD | BS_AUTOCHECKBOX,	// window style
    2 + 52 + 4,	// horizontal position of window
    2,	// vertical position of window
    68,	// window width
    24,	// window height
    hKeybdWnd,	// handle to parent or owner window
    NULL,	// handle to menu or child-window identifier
    hInst,	// handle to application instance
    NULL 	// pointer to window-creation data
   );

	hKeybdDC = GetDC(hKeybdWnd);
	hKeyErasePen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
	hKeyDrawPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));

	return hKeybdWnd;
}

