 // ***********************************************************
// Use this code at your own risk.
//
// Direct questions and/or fixes to:
//   Illah Nourbakhsh     illah@cs.cmu.edu
//   Kevin Wilson         kwilson@cs.cmu.edu
//   Carnegie Mellon University Robotics Institute
//
// Last updated:  June 4, 1998 (SCOUT compatible)
//
// ***********************************************************

// interface to robot through serial port

#include <windows.h>
#include <stdlib.h>
#include <winuser.h>

#include "include\interface.h"
#include "include\interfaceprivate.h"
#include "include\resource.h"

const int MaxLines = 100;
// maximum number of lines in the output box.

SensorType Sensors; // the global state that the user looks at.

extern HWND hWndGlobal, hConsoleWin;
extern HANDLE hInst;
extern char szAppName[], LineBuf[];  // LineBuf holds the contents of the
                                     // input box.
extern CRITICAL_SECTION Input1Lock,  // Input1Lock and Input2Lock are used
	Input2Lock,                  // for input box synchronization.
	PrintLock,  // serialize OutputText calls
	SerialLock, // serialize serial port access
	SonarLock;  // serialize changes to sonar state

extern int LineAvailable, InputBarrier, LineWait;  // input box synchronization

extern int UpArrow, DownArrow, LeftArrow, RightArrow, SpaceBar, EnterKey,
	ZeroNow, SonarState, FooState, BarState;
// a bunch of user input state that is set in generic.c

HANDLE hCom1;  // serial port handle
HWND hSonarWnd, hZeroButton, hSonarButton, hKeybdWnd;
int SonarsActive = 0, SonarsHidden = 0, SonarWinShown = 0,
	SonarActive[NUMSONARS], KeybdVisible = 0;
// state that is used to determine what should be drawn.

// clears input box
void ClearInputBox()
{
	SetDlgItemText(hConsoleWin, IDC_EDIT2, "");
}

// copies contents of input box into buf.  blocks until <Enter> is hit.
void GetInputBoxText(char *buf)
{
	int done;

	EnterCriticalSection(&Input1Lock);
	LineWait = 1;
	LineAvailable = 0;
	LeaveCriticalSection(&Input1Lock);

	done = 0;
	while (!done) {
		Sleep(0);
		EnterCriticalSection(&Input1Lock);
		done = LineAvailable;
		LeaveCriticalSection(&Input1Lock);
	}

	rn2n(buf, LineBuf);

	done = 0;
	while (!done) {
		EnterCriticalSection(&Input2Lock);
		done = (InputBarrier == 0);
		LeaveCriticalSection(&Input2Lock);
	}

	EnterCriticalSection(&Input2Lock);
	InputBarrier = 1;
	LeaveCriticalSection(&Input2Lock);

	done = 0;
	while (!done) {
		EnterCriticalSection(&Input2Lock);
		done = (InputBarrier == 0);
		LeaveCriticalSection(&Input2Lock);
	}
}

// clears input box and gets the next line that is typed.
void GetLineInputBox(char *buf)
{
	ClearInputBox();
	GetInputBoxText(buf);
}

// returns 1 if Foo clicked since last call.
int GetFooClick()
{
	// FooState is set to 1 by generic.c
	if (FooState) {
		FooState = 0;
		return 1;
	} else
		return 0;
	
}

// returns 1 if Bar clicked since last call.
int GetBarClick()
{
	// BarState is set to 1 by generic.c
	if (BarState) {
		BarState = 0;
		return 1;
	} else
		return 0;

}

// returns 1 if left mouse button clicked since last call, 0 otherwise.
int GetLeftClick()
{
	return !(GetAsyncKeyState(VK_LBUTTON) == 0);
}

// returns 1 if right mouse button clicked since last call, 0 otherwise.
int GetRightClick()
{
	return !(GetAsyncKeyState(VK_RBUTTON) == 0);
}


// open serial port at 9600 baud with other appropriate parameters.
// returns:
// 0 on success
// -1 on failure to open serial port
// -2 on failure to read port state
// -3 on failure to set port state
int OpenSerial()
{
	DCB dcb;	
	HANDLE hCom;
	DWORD dwError;
	BOOL fSuccess;

	EnterCriticalSection(&SerialLock);
	hCom = CreateFile("COM1",
		GENERIC_READ | GENERIC_WRITE,
		0,    /* comm devices must be opened w/exclusive-access */
		NULL, /* no security attrs */
		OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
		0,    /* not overlapped I/O */
		NULL  /* hTemplate must be NULL for comm devices */
		);

	if (hCom == INVALID_HANDLE_VALUE) {
		dwError = GetLastError();
		LeaveCriticalSection(&SerialLock);

		return (-1);
	}

	/*
	 * Omit the call to SetupComm to use the default queue sizes.
	 * Get the current configuration.
	*/

	fSuccess = GetCommState(hCom, &dcb);

	if (!fSuccess) {
		LeaveCriticalSection(&SerialLock);
		return (-2);
	}

	/* Fill in the DCB: baud=19200, 8 data bits, no parity, 1 stop bit. */

	dcb.BaudRate = 38400; // changed 6/4/98 illah
	dcb.fBinary = TRUE;
	dcb.fParity = FALSE;
	
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;

	fSuccess = SetCommState(hCom, &dcb);

	if (!fSuccess) {
		LeaveCriticalSection(&SerialLock);	
		return (-3);
	}

	hCom1 = hCom;
	LeaveCriticalSection(&SerialLock);
	return (0);
}

// issue Zero command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
int Zero()
{
    char buf[50];
	int i = 0;
	int retval;
	
	Append2Packet(buf, &i, 1); // start byte
	Append2Packet(buf, &i, 2); // low byte of packet length
	Append2Packet(buf, &i, 0); // high byte of packet length
	Append2Packet(buf, &i, 0xd); // zero code
	Append2Packet(buf, &i, 16); // checksum
	Append2Packet(buf, &i, 92); // end byte

	retval = RobotCommand(buf, i, 30000);
//	sprintf(buf, "Zero:%d\n", retval);
//	OutputText(buf);

	return retval; 
}

// issue Limp command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int Limp()
{
    char buf[50];
	int i = 0;
	int retval;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 2);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0xa);
	Append2Packet(buf, &i, 13);
	Append2Packet(buf, &i, 92);

	retval = RobotCommand(buf, i, 4000);
//	sprintf(buf, "Limp:%d\n", retval);
//	OutputText(buf);

	return retval; 
}

// issue ConfAc command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int ConfAc(int ta, int sa, int ra)
{
    char buf[50];
	int i = 0;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 8);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x1);
	Append2Packet(buf, &i, LOWBYTESIGNED(ta));
	Append2Packet(buf, &i, SECONDBYTESIGNED(ta));
	Append2Packet(buf, &i, LOWBYTESIGNED(sa));
	Append2Packet(buf, &i, SECONDBYTESIGNED(sa));
	Append2Packet(buf, &i, LOWBYTESIGNED(ra));
	Append2Packet(buf, &i, SECONDBYTESIGNED(ra));
	Append2Packet(buf, &i, (unsigned char) ((1 + 8 + 0 + 0x1 + (unsigned) LOWBYTESIGNED(ta) 
		+ (unsigned) SECONDBYTESIGNED(ta) 
		+ (unsigned) LOWBYTESIGNED(sa) 
		+ (unsigned) SECONDBYTESIGNED(sa) 
		+ (unsigned) LOWBYTESIGNED(ra) 
		+ (unsigned) SECONDBYTESIGNED(ra)) % 256)); 
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// issue ConfTimeout command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int ConfTimeout(int timeout)
{
    char buf[50];
	int i = 0;
	
	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 4);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x12);
	Append2Packet(buf, &i, LOWBYTEUNSIGNED(timeout));
	Append2Packet(buf, &i, SECONDBYTEUNSIGNED(timeout));
	Append2Packet(buf, &i, (unsigned char) ((1 + 4 + 0 + 0x12 + LOWBYTEUNSIGNED(timeout) +
			  SECONDBYTEUNSIGNED(timeout)) % 256)); 
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// issue GS command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int GS()
{
    char buf[50];
	int i = 0;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 2);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x7);
	Append2Packet(buf, &i, 10);
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int RMove(int rightspeed, int leftspeed)
{
	return VM(rightspeed, leftspeed, 0);
}

// issue VM command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int VM(int rightvel, int leftvel, int unused)
{
    char buf[50];
	int i = 0;
	int retval;

	if (rightvel > 350)
		rightvel = 350;
	else if (rightvel < -350)
		rightvel = -350;
	if (leftvel > 350)
		leftvel = 350;
	else if (leftvel < -350)
		leftvel = -350;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 8);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x5);
	Append2Packet(buf, &i, LOWBYTESIGNED(rightvel));
	Append2Packet(buf, &i, SECONDBYTESIGNED(rightvel));
	Append2Packet(buf, &i, LOWBYTESIGNED(leftvel));
	Append2Packet(buf, &i, SECONDBYTESIGNED(leftvel));
	Append2Packet(buf, &i, LOWBYTESIGNED(unused));
	Append2Packet(buf, &i, SECONDBYTESIGNED(unused));
	Append2Packet(buf, &i, (char) ((14 /* 1 + 8 + 0x5 */
			  + LOWBYTESIGNED(rightvel) + SECONDBYTESIGNED(rightvel) + 
			    LOWBYTESIGNED(leftvel) + SECONDBYTESIGNED(leftvel)
			  + LOWBYTESIGNED(unused) + SECONDBYTESIGNED(unused)) & 0xff));
	Append2Packet(buf, &i, 92);

	retval = RobotCommand(buf, i, 4000);
//	sprintf(buf, "VM:%d\n", retval);
//	OutputText(buf);

	return retval; 
}

// issue ConfSonars command.
//
// rate is the delay between sonar firings.
// orderlist is an array of sonar numbers.
// listsize is the number of elements in orderlist.
//
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
// also returns -30 for invalid listsize and -31 for invalid sonar numbers
// in orderlist.
// put a 255 after the last sonar you want to fire, then fill out orderlist to
// 16 elements with zeroes.
int ConfSonars(int rate, int orderlist[])
{
    char buf[50];
    int result, i, j, sum, size=0;

	
	for (i=0;i<16;i++) {
		if (((orderlist[i] < 0) || (orderlist[i] >= NUMSONARS)) &&
			(orderlist[i] != 255))
			return -31;
	}

	Append2Packet(buf, &size, 1);
	Append2Packet(buf, &size, 19);
	Append2Packet(buf, &size, 0);
	Append2Packet(buf, &size, 0xF);
	Append2Packet(buf, &size, (unsigned char) rate);
	sum = 1 + 19 + 0xF + rate;
	for (i=0; i < 16; i++) {
		Append2Packet(buf, &size, (unsigned char) orderlist[i]);
		sum += orderlist[i];
	}

	Append2Packet(buf, &size, (char) (sum & 0xFF));
	Append2Packet(buf, &size, 92);

	EnterCriticalSection(&SonarLock);
	
	if (result = RobotCommand(buf, size, 4000)) {
		LeaveCriticalSection(&SonarLock);
		return result;
	}

	if ((1 > 0) && (orderlist[0] != 127))
		SonarsActive = 1;
	else
		SonarsActive = 0;

	for (i=0;i<NUMSONARS;i++) {
		SonarActive[i] = 0;
	}
	for (i=0;i<16;i++) {
		SonarActive[orderlist[i]] = 1;
	}

	if (!SonarsHidden && !SonarWinShown && SonarsActive) {
		ShowWindow(hSonarWnd, SW_SHOW);
		SonarWinShown = 1;
	} else if (SonarWinShown && !SonarsActive) {
		ShowWindow(hSonarWnd, SW_HIDE);
		SonarWinShown = 0;
	}

	LeaveCriticalSection(&SonarLock);
	return 0;
	
}

// turn all sonars on
int SonarsOn()
{
	int orderlist[16] = {0,3,6,9,12,15,2,5,8,11,14,1,4,7,10,13}; 
	int retval;
//	char buf[256];

	retval = ConfSonars(2, orderlist);		
//	sprintf(buf, "SonarsOn:%d\n", retval);
//	OutputText(buf);
	return retval;
}

// turn all sonars off
int SonarsOff()
{
	int orderlist[16] = {255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; 
	int retval;
//	char buf[256];
	
	retval = ConfSonars(0, orderlist);
//	sprintf(buf, "SonarsOff:%d\n", retval);
//	OutputText(buf);
	return retval;
}

// flip sonar display state
void SonarHideToggle()
{
	if (SonarsHidden)
		UnhideSonars();
	else
		HideSonars();
	
}

// inhibits drawing of sonar display
void HideSonars()
{
	HWND hCheck;

	EnterCriticalSection(&SonarLock);
	
	if (SonarWinShown) {
		ShowWindow(hSonarWnd, SW_HIDE);
		SonarWinShown = 0;
	}

	SonarsHidden = 1;
	hCheck = GetDlgItem(hConsoleWin, IDC_SONARCHECK);
	SendMessage(hCheck, BM_SETCHECK, BST_UNCHECKED, 0);

	LeaveCriticalSection(&SonarLock);

}

// allow drawing of sonar display
void UnhideSonars()
{
	HWND hCheck;

	EnterCriticalSection(&SonarLock);

	if (SonarsActive && !SonarWinShown) {
		ShowWindow(hSonarWnd, SW_SHOW);
		SonarWinShown = 1;
	}

	SonarsHidden = 0;
	hCheck = GetDlgItem(hConsoleWin, IDC_SONARCHECK);
	SendMessage(hCheck, BM_SETCHECK, BST_CHECKED, 0);

	LeaveCriticalSection(&SonarLock);
}

// open Keyboard Control window which allows user to use keyboard to
// control robot.
// returns 1 if sonars are on when it returns, 0 if they are off.
int KeybdControl()
{
	int tv=0, rv=0, oldsonar;
	int onlist[16] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15},
		offlist[16] = {255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	UpArrow = DownArrow = LeftArrow = RightArrow = ZeroNow = SpaceBar =
		EnterKey = 0;
	
	SonarState = oldsonar = SonarsActive;
	SendMessage(hSonarButton, BM_SETCHECK,
				(SonarsActive)?BST_CHECKED:BST_UNCHECKED, 0);
	UnhideKeybd();
	SetForegroundWindow(hKeybdWnd);
	
	while(KeybdVisible) {
		if (UpArrow) {
			tv += 350/25;
			UpArrow = UpArrow & 1;
		}
		if (DownArrow) {
			tv -= 350/25;
			DownArrow = DownArrow & 1;
		}
		if (LeftArrow) {
			rv += 350/25;
			LeftArrow = LeftArrow & 1;
		}
		if (RightArrow) {
			rv -= 350/25;
			RightArrow = RightArrow & 1;
		}
		if (EnterKey) {
			tv=rv=0;
			EnterKey = EnterKey & 1;
		}
		if (INTABS(tv) > 350)
			tv = PSIGN(tv) * 350;
		if (INTABS(rv) > 350)
			rv = PSIGN(rv) * 350;

		UpdateKeybdWindow(tv, rv);
		
		if (SpaceBar) {
			RMove(tv+rv, tv-rv);
		//	RMove(tv,rv*PSIGN(tv));
			SpaceBar = SpaceBar & 1;
		} else
			Limp();

		if (SonarState != oldsonar) {
			oldsonar = SonarState;
			if (SonarState) {
				SonarsOn();
			} else {
				SonarsOff();
			}
		}
		if (ZeroNow) {
			Zero();
			ZeroNow = 0;
		}		
	}

	return SonarsActive;
}

// play wav file fname.
// returns 1 on success and 0 on failure.
BOOL Speak(char *fname)
{
	return PlaySound(fname, NULL, SND_ASYNC | SND_FILENAME);
}

// output buf to Output box in Robot Console.
// silently truncates if the string would cause a buffer overflow.
void OutputText(char *buf)		
{
	static struct TextBufType statBuffer;
	struct TextBufType *statBuf;
	HWND hEditWnd;
	SCROLLINFO sinf;
	int oldlen, oflow, buflen, lastc;
	char tempc;
	
	EnterCriticalSection(&PrintLock);
	
	statBuf = &statBuffer;
	
	if (statBuf->numLines >= MaxLines) {
		// alternate between 2 buffers, scrolling old stuff off the top.
		strcpy(statBuf->b[1-statBuf->currBuf],
			SkipLine(statBuf->b[statBuf->currBuf]));
		statBuf->currBuf = 1 - statBuf->currBuf;
	} else {
		statBuf->numLines++;
	}

	oldlen = strlen(statBuf->b[statBuf->currBuf]);
	buflen = strlen(buf);
	oflow = (oldlen + 2 * buflen) - MaxChars;
	if (oflow > 0) {
		lastc = (MaxChars - oldlen) / 2 - 1;
		tempc = buf[lastc];
		buf[lastc] = 0;
	}
	// truncate if overflow is possible
	
	n2rn(&(statBuf->b[statBuf->currBuf]
		   [strlen(statBuf->b[statBuf->currBuf])]), buf);
	if (oflow > 0) {
		buf[lastc] = tempc;
	}		
	
	SetDlgItemText(hConsoleWin, IDC_EDIT1, statBuf->b[statBuf->currBuf]);	
	
	hEditWnd = GetDlgItem(hConsoleWin,IDC_EDIT1);
	
	sinf.cbSize = sizeof(SCROLLINFO);
	sinf.nPos = 9999;
	sinf.fMask = SIF_POS;
	
	SendDlgItemMessage(hConsoleWin, IDC_EDIT1, EM_LINESCROLL, 0, 9999);
	// scroll to bottom
	
	LeaveCriticalSection(&PrintLock);
}

